@@ -1,3 +1,5 @@
+#include "atari.h"
+
 #define FALSE 0
 #define TRUE 1
 
@@ -17,6 +19,11 @@ int Atari_Exit (int run_monitor)
   return restart;
 }
 
+int Atari_Keyboard (void)
+{
+  return AKEY_NONE;
+}
+
 int Atari_PORT (int num)
 {
   return 0xff;
