@@ -64,6 +64,7 @@ int SIO_Mount (int diskno, char *filename)
       if (status == -1)
 	{
 	  perror ("SIO_Mount");
+	  Atari800_Exit (FALSE);
 	  exit (1);
 	}
 
@@ -107,17 +108,17 @@ SIO ()
 {
   CPU_Status	cpu_status;
 
-  UBYTE DDEVIC = GetByte (0x0300);
-  UBYTE DUNIT = GetByte (0x0301);
-  UBYTE DCOMND = GetByte (0x0302);
-  UBYTE DSTATS = GetByte(0x0303);
-  UBYTE DBUFLO = GetByte(0x0304);
-  UBYTE DBUFHI = GetByte(0x0305);
-  UBYTE DTIMLO = GetByte(0x0306);
-  UBYTE DBYTLO = GetByte(0x0308);
-  UBYTE DBYTHI = GetByte(0x0309);
-  UBYTE DAUX1 = GetByte(0x030a);
-  UBYTE DAUX2 = GetByte(0x030b);
+  UBYTE DDEVIC = memory[0x0300];
+  UBYTE DUNIT = memory[0x0301];
+  UBYTE DCOMND = memory[0x0302];
+  UBYTE DSTATS = memory[0x0303];
+  UBYTE DBUFLO = memory[0x0304];
+  UBYTE DBUFHI = memory[0x0305];
+  UBYTE DTIMLO = memory[0x0306];
+  UBYTE DBYTLO = memory[0x0308];
+  UBYTE DBYTHI = memory[0x0309];
+  UBYTE DAUX1 = memory[0x030a];
+  UBYTE DAUX2 = memory[0x030b];
 
   int	sector;
   int	buffer;
@@ -147,6 +148,7 @@ SIO ()
 	  break;
 	default :
 	  printf ("Fatal Error in atari_sio.c\n");
+	  Atari800_Exit (FALSE);
 	  exit (1);
 	}
 
@@ -199,9 +201,9 @@ SIO ()
 	  for (i=0;i<count;i++)
 	    {
 	      if (sectorsize[DUNIT-1] == 256)
-		PutByte (buffer+i, 32 + 16)
+		memory[buffer+i] = 32 + 16;
 	      else
-		PutByte (buffer+i, 16)
+		memory[buffer+i] = 16;
 	    }
 	  cpu_status.Y = 1;
 	  cpu_status.flag.N = 0;
@@ -221,5 +223,149 @@ SIO ()
 
   CPU_PutStatus (&cpu_status);
 
-  PutByte(0x0303, cpu_status.Y);
+  memory[0x0303] = cpu_status.Y;
 }
+
+static unsigned char cmd_frame[5];
+static int ncmd = 0;
+static int checksum = 0;
+
+static unsigned char data[128];
+static int offst;
+
+static int buffer_offset;
+static int buffer_size;
+
+void Command_Frame (void)
+{
+  switch (cmd_frame[1])
+    {
+    case 'R' : /* Read */
+      printf ("Read command\n");
+      break;
+    case 'W' : /* Write with verify */
+      printf ("Write command\n");
+      break;
+    case 'S' : /* Status */
+      printf ("Status command\n");
+      data[0] = 0x41; /* ACK */
+      data[1] = 0x10;
+      data[2] = 0x10;
+      data[3] = 0x10;
+      data[4] = 0x10;
+      data[5] = 0x40; /* Checksum */
+      data[6] = 0x43; /* COMPLETE */
+      buffer_offset = 0;
+      buffer_size = 7;
+      break;
+    case 'P' : /* Put without verify */
+      printf ("Put command\n");
+      break;
+    case '!' : /* Format */
+      printf ("Format command\n");
+      break;
+    case 'T' : /* Read Address */
+      printf ("Read Address command\n");
+      break;
+    case 'Q' : /* Read Spin */
+      printf ("Read Spin command\n");
+      break;
+    case 'U' : /* Motor On */
+      printf ("Motor On command\n");
+      break;
+    case 'V' : /* Verify Sector */
+      printf ("Verify Sector\n");
+      break;
+    default :
+      printf ("Unknown command\n");
+      break;
+  }
+}
+
+void SIO_SEROUT (unsigned char byte, int cmd)
+{
+  checksum += (unsigned char)byte;
+  while (checksum > 255)
+    checksum = checksum - 255;
+
+#ifdef DEBUG
+  printf ("SIO_SEROUT: byte = %x, checksum = %x, cmd = %d\n",
+	  byte, checksum, cmd);
+#endif
+
+  if (cmd)
+    {
+      cmd_frame[ncmd++] = byte;
+      if (ncmd == 5)
+	{
+	  int sector;
+
+	  Command_Frame ();
+/*
+	  sector = cmd_frame[2] + cmd_frame[3] * 256;
+	  printf ("Sector: %d(%x)\n", sector, sector);
+	  lseek (disk[0], (sector-1)*128, SEEK_SET);
+	  read (disk[0], data, 128);
+*/
+	  offst = 0;
+	  checksum = 0;
+/*
+	  IRQST &= 0xdf;
+*/
+/*
+	  IRQST &= 0xc7;
+*/
+	  IRQST &= ~(IRQEN & (0x10 | 0x08));
+	  INTERRUPT |= IRQ_MASK;
+	  ncmd = 0;
+	}
+      else
+	{
+/*
+	  IRQST &= 0xe7;
+*/
+	  IRQST &= ~(IRQEN & (0x10 | 0x08));
+	  INTERRUPT |= IRQ_MASK;
+	}
+
+      if (cmd_frame[0] == 0)
+	ncmd = 0;
+    }
+  else
+    {
+      ncmd = 0;
+/*
+      IRQST &= 0xe7;
+*/
+      IRQST &= ~(IRQEN & (0x10 | 0x80));
+      INTERRUPT |= IRQ_MASK;
+    }
+}
+
+int SIO_SERIN ()
+{
+  int byte;
+
+  if (buffer_offset < buffer_size)
+    {
+      byte = (int)data[buffer_offset++];
+
+#ifdef DEBUG
+      printf ("SERIN: byte = %x\n", byte);
+#endif
+
+      if (buffer_offset < buffer_size)
+	{
+/*
+	  IRQST &= 0xdf;
+*/
+	  IRQST &= ~(IRQEN & 0x20);
+	  INTERRUPT |= IRQ_MASK;
+	  printf ("Setting SERIN Interrupt again\n");
+	}
+    }
+
+  return byte;
+}
+
+
