@@ -44,6 +44,8 @@ typedef enum
 
 static WindowSize windowsize = Large;
 
+static int x11bug = FALSE;
+
 static int window_width;
 static int window_height;
 
@@ -142,16 +144,21 @@ int GetKeyCode (XEvent *event)
 	{
 	case XK_Shift_L :
 	case XK_Shift_R :
-	  SHIFT = 0x40;
+	  SHIFT = AKEY_SHFT;
 	  break;
 	case XK_Control_L :
 	case XK_Control_R :
-	  CONTROL = 0x80;
+	  CONTROL = AKEY_CTRL;
 	  break;
 	case XK_Caps_Lock :
-	  keycode = AKEY_CAPSTOGGLE;
+	  if (SHIFT)
+	    keycode = AKEY_CAPSLOCK;
+	  else
+	    keycode = AKEY_CAPSTOGGLE;
 	  break;
 	case XK_Shift_Lock :
+	  if (x11bug)
+	    printf ("XK_Shift_Lock\n");
 	  break;
 	case XK_Alt_L :
 	case XK_Alt_R :
@@ -232,91 +239,91 @@ int GetKeyCode (XEvent *event)
 	    keycode = AKEY_TAB;
 	  break;
 	case XK_exclam :
-	  keycode = '!';
+	  keycode = AKEY_EXCLAMATION;
 	  break;
 	case XK_quotedbl :
-	  keycode = '"';
+	  keycode = AKEY_DBLQUOTE;
 	  break;
 	case XK_numbersign :
-	  keycode = '#';
+	  keycode = AKEY_HASH;
 	  break;
 	case XK_dollar :
-	  keycode = '$';
+	  keycode = AKEY_DOLLAR;
 	  break;
 	case XK_percent :
-	  keycode = '%';
+	  keycode = AKEY_PERCENT;
 	  break;
 	case XK_ampersand :
-	  keycode = '&';
+	  keycode = AKEY_AMPERSAND;
 	  break;
 	case XK_quoteright :
-	  keycode = '\'';
+	  keycode = AKEY_QUOTE;
 	  break;
 	case XK_at :
-	  keycode = '@';
+	  keycode = AKEY_AT;
 	  break;
 	case XK_parenleft :
-	  keycode = '(';
+	  keycode = AKEY_PARENLEFT;
 	  break;
 	case XK_parenright :
-	  keycode = ')';
+	  keycode = AKEY_PARENRIGHT;
 	  break;
 	case XK_less :
-	  keycode = '<';
+	  keycode = AKEY_LESS;
 	  break;
 	case XK_greater :
-	  keycode = '>';
+	  keycode = AKEY_GREATER;
 	  break;
 	case XK_equal :
-	  keycode = '=';
+	  keycode = AKEY_EQUAL;
 	  break;
 	case XK_question :
-	  keycode = '?';
+	  keycode = AKEY_QUESTION;
 	  break;
 	case XK_minus :
-	  keycode = '-';
+	  keycode = AKEY_MINUS;
 	  break;
 	case XK_plus :
-	  keycode = '+';
+	  keycode = AKEY_PLUS;
 	  break;
 	case XK_asterisk :
-	  keycode = '*';
+	  keycode = AKEY_ASTERISK;
 	  break;
 	case XK_slash :
-	  keycode = '/';
+	  keycode = AKEY_SLASH;
 	  break;
 	case XK_colon :
-	  keycode = ':';
+	  keycode = AKEY_COLON;
 	  break;
 	case XK_semicolon :
-	  keycode = ';';
+	  keycode = AKEY_SEMICOLON;
 	  break;
 	case XK_comma :
-	  keycode = ',';
+	  keycode = AKEY_COMMA;
 	  break;
 	case XK_period :
-	  keycode = '.';
+	  keycode = AKEY_FULLSTOP;
 	  break;
 	case XK_underscore :
-	  keycode = '_';
+	  keycode = AKEY_UNDERSCORE;
 	  break;
 	case XK_bracketleft :
-	  keycode = '[';
+	  keycode = AKEY_BRACKETLEFT;
 	  break;
 	case XK_bracketright :
-	  keycode = ']';
+	  keycode = AKEY_BRACKETRIGHT;
 	  break;
 	case XK_asciicircum :
-	  keycode = '^';
+	  keycode = AKEY_CIRCUMFLEX;
 	  break;
 	case XK_backslash :
-	  keycode = '\\';
+	  keycode = AKEY_BACKSLASH;
 	  break;
 	case XK_bar :
-	  keycode = '|';
+	  keycode = AKEY_BAR;
 	  break;
 	case XK_space :
-	  keycode = ' ';
+	  keycode = AKEY_SPACE;
 	  keypad_trig = 0;
 	  break;
 	case XK_Return :
@@ -324,480 +331,112 @@ int GetKeyCode (XEvent *event)
 	  keypad_stick = STICK_CENTRE;
 	  break;
 	case XK_0 :
-	  if (CONTROL)
-	    keycode = AKEY_CTRL_0;
-	  else
-	    keycode = '0';
+	  keycode = CONTROL | AKEY_0;
 	  break;
 	case XK_1 :
-	  if (CONTROL)
-	    keycode = AKEY_CTRL_1;
-	  else
-	    keycode = '1';
+	  keycode = CONTROL | AKEY_1;
 	  break;
 	case XK_2 :
-	  if (CONTROL)
-	    keycode = AKEY_CTRL_2;
-	  else
-	    keycode = '2';
+	  keycode = CONTROL | AKEY_2;
 	  break;
 	case XK_3 :
-	  if (CONTROL)
-	    keycode = AKEY_CTRL_3;
-	  else
-	    keycode = '3';
+	  keycode = CONTROL | AKEY_3;
 	  break;
 	case XK_4 :
-	  if (CONTROL)
-	    keycode = AKEY_CTRL_4;
-	  else
-	    keycode = '4';
+	  keycode = CONTROL | AKEY_4;
 	  break;
 	case XK_5 :
-	  if (CONTROL)
-	    keycode = AKEY_CTRL_5;
-	  else
-	    keycode = '5';
+	  keycode = CONTROL | AKEY_5;
 	  break;
 	case XK_6 :
-	  if (CONTROL)
-	    keycode = AKEY_CTRL_6;
-	  else
-	    keycode = '6';
+	  keycode = CONTROL | AKEY_6;
 	  break;
 	case XK_7 :
-	  if (CONTROL)
-	    keycode = AKEY_CTRL_7;
-	  else
-	    keycode = '7';
+	  keycode = CONTROL | AKEY_7;
 	  break;
 	case XK_8 :
-	  if (CONTROL)
-	    keycode = AKEY_CTRL_8;
-	  else
-	    keycode = '8';
+	  keycode = CONTROL | AKEY_8;
 	  break;
 	case XK_9 :
-	  if (CONTROL)
-	    keycode = AKEY_CTRL_9;
-	  else
-	    keycode = '9';
+	  keycode = CONTROL | AKEY_9;
 	  break;
-	case XK_a :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_A;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_A;
-	  else
-	    keycode = 'a';
+	case XK_A : case XK_a :
+	  keycode = SHIFT | CONTROL | AKEY_a;
 	  break;
-	case XK_b :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_B;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_B;
-	  else
-	    keycode = 'b';
+	case XK_B : case XK_b :
+	  keycode = SHIFT | CONTROL | AKEY_b;
 	  break;
-	case XK_c :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_C;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_C;
-	  else
-	    keycode = 'c';
+	case XK_C : case XK_c :
+	  keycode = SHIFT | CONTROL | AKEY_c;
 	  break;
-	case XK_d :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_D;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_D;
-	  else
-	    keycode = 'd';
+	case XK_D : case XK_d :
+	  keycode = SHIFT | CONTROL | AKEY_d;
 	  break;
-	case XK_e :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_E;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_E;
-	  else
-	    keycode = 'e';
+	case XK_E : case XK_e :
+	  keycode = SHIFT | CONTROL | AKEY_e;
 	  break;
-	case XK_f :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_F;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_F;
-	  else
-	    keycode = 'f';
+	case XK_F : case XK_f :
+	  keycode = SHIFT | CONTROL | AKEY_f;
 	  break;
-	case XK_g :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_G;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_G;
-	  else
-	    keycode = 'g';
+	case XK_G : case XK_g :
+	  keycode = SHIFT | CONTROL | AKEY_g;
 	  break;
-	case XK_h :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_H;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_H;
-	  else
-	    keycode = 'h';
+	case XK_H : case XK_h :
+	  keycode = SHIFT | CONTROL | AKEY_h;
 	  break;
-	case XK_i :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_I;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_I;
-	  else
-	    keycode = 'i';
+	case XK_I : case XK_i :
+	  keycode = SHIFT | CONTROL | AKEY_i;
 	  break;
-	case XK_j :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_J;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_J;
-	  else
-	    keycode = 'j';
+	case XK_J : case XK_j :
+	  keycode = SHIFT | CONTROL | AKEY_j;
 	  break;
-	case XK_k :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_K;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_K;
-	  else
-	    keycode = 'k';
+	case XK_K : case XK_k :
+	  keycode = SHIFT | CONTROL | AKEY_k;
 	  break;
-	case XK_l :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_L;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_L;
-	  else
-	    keycode = 'l';
+	case XK_L : case XK_l :
+	  keycode = SHIFT | CONTROL | AKEY_l;
 	  break;
-	case XK_m :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_M;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_M;
-	  else
-	    keycode = 'm';
+	case XK_M : case XK_m :
+	  keycode = SHIFT | CONTROL | AKEY_m;
 	  break;
-	case XK_n :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_N;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_N;
-	  else
-	    keycode = 'n';
+	case XK_N : case XK_n :
+	  keycode = SHIFT | CONTROL | AKEY_n;
 	  break;
-	case XK_o :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_O;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_O;
-	  else
-	    keycode = 'o';
+	case XK_O : case XK_o :
+	  keycode = SHIFT | CONTROL | AKEY_o;
 	  break;
-	case XK_p :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_P;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_P;
-	  else
-	    keycode = 'p';
+	case XK_P : case XK_p :
+	  keycode = SHIFT | CONTROL | AKEY_p;
 	  break;
-	case XK_q :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_Q;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_Q;
-	  else
-	    keycode = 'q';
+	case XK_Q : case XK_q :
+	  keycode = SHIFT | CONTROL | AKEY_q;
 	  break;
-	case XK_r :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_R;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_R;
-	  else
-	    keycode = 'r';
+	case XK_R : case XK_r :
+	  keycode = SHIFT | CONTROL | AKEY_r;
 	  break;
-	case XK_s :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_S;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_S;
-	  else
-	    keycode = 's';
+	case XK_S : case XK_s :
+	  keycode = SHIFT | CONTROL | AKEY_s;
 	  break;
-	case XK_t :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_T;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_T;
-	  else
-	    keycode = 't';
+	case XK_T : case XK_t :
+	  keycode = SHIFT | CONTROL | AKEY_t;
 	  break;
-	case XK_u :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_U;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_U;
-	  else
-	    keycode = 'u';
+	case XK_U : case XK_u :
+	  keycode = SHIFT | CONTROL | AKEY_u;
 	  break;
-	case XK_v :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_V;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_V;
-	  else
-	    keycode = 'v';
+	case XK_V : case XK_v :
+	  keycode = SHIFT | CONTROL | AKEY_v;
 	  break;
-	case XK_w :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_W;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_W;
-	  else
-	    keycode = 'w';
+	case XK_W : case XK_w :
+	  keycode = SHIFT | CONTROL | AKEY_w;
 	  break;
-	case XK_x :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_X;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_X;
-	  else
-	    keycode = 'x';
+	case XK_X : case XK_x :
+	  keycode = SHIFT | CONTROL | AKEY_x;
 	  break;
-	case XK_y :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_Y;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_Y;
-	  else
-	    keycode = 'y';
+	case XK_Y : case XK_y :
+	  keycode = SHIFT | CONTROL | AKEY_y;
 	  break;
-	case XK_z :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_Z;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_Z;
-	  else
-	    keycode = 'z';
-	  break;
-	case XK_A :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_A;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_A;
-	  else
-	    keycode = 'A';
-	  break;
-	case XK_B :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_B;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_B;
-	  else
-	    keycode = 'B';
-	  break;
-	case XK_C :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_C;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_C;
-	  else
-	    keycode = 'C';
-	  break;
-	case XK_D :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_D;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_D;
-	  else
-	    keycode = 'D';
-	  break;
-	case XK_E :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_E;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_E;
-	  else
-	    keycode = 'E';
-	  break;
-	case XK_F :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_F;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_F;
-	  else
-	    keycode = 'F';
-	  break;
-	case XK_G :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_G;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_G;
-	  else
-	    keycode = 'G';
-	  break;
-	case XK_H :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_H;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_H;
-	  else
-	    keycode = 'H';
-	  break;
-	case XK_I :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_I;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_I;
-	  else
-	    keycode = 'I';
-	  break;
-	case XK_J :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_J;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_J;
-	  else
-	    keycode = 'J';
-	  break;
-	case XK_K :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_K;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_K;
-	  else
-	    keycode = 'K';
-	  break;
-	case XK_L :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_L;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_L;
-	  else
-	    keycode = 'L';
-	  break;
-	case XK_M :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_M;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_M;
-	  else
-	    keycode = 'M';
-	  break;
-	case XK_N :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_N;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_N;
-	  else
-	    keycode = 'N';
-	  break;
-	case XK_O :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_O;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_O;
-	  else
-	    keycode = 'O';
-	  break;
-	case XK_P :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_P;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_P;
-	  else
-	    keycode = 'P';
-	  break;
-	case XK_Q :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_Q;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_Q;
-	  else
-	    keycode = 'Q';
-	  break;
-	case XK_R :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_R;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_R;
-	  else
-	    keycode = 'R';
-	  break;
-	case XK_S :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_S;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_S;
-	  else
-	    keycode = 'S';
-	  break;
-	case XK_T :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_T;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_T;
-	  else
-	    keycode = 'T';
-	  break;
-	case XK_U :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_U;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_U;
-	  else
-	    keycode = 'U';
-	  break;
-	case XK_V :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_V;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_V;
-	  else
-	    keycode = 'V';
-	  break;
-	case XK_W :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_W;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_W;
-	  else
-	    keycode = 'W';
-	  break;
-	case XK_X :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_X;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_X;
-	  else
-	    keycode = 'X';
-	  break;
-	case XK_Y :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_Y;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_Y;
-	  else
-	    keycode = 'Y';
-	  break;
-	case XK_Z :
-	  if (SHIFT && CONTROL)
-	    keycode = AKEY_SHFTCTRL_Z;
-	  else if (CONTROL)
-	    keycode = AKEY_CTRL_Z;
-	  else
-	    keycode = 'Z';
+	case XK_Z : case XK_z :
+	  keycode = SHIFT | CONTROL | AKEY_z;
 	  break;
 	case XK_KP_0 :
 	  keypad_trig = 0;
@@ -830,9 +469,11 @@ int GetKeyCode (XEvent *event)
 	  keypad_stick = STICK_UR;
 	  break;
 	default :
-	  printf ("Pressed Keysym = %x\n", (int)keysym);
+	  if (x11bug)
+	    printf ("Pressed Keysym = %x\n", (int)keysym);
 	  break;
 	}
+      break;
     case KeyRelease :
       switch (keysym)
 	{
@@ -844,6 +485,16 @@ int GetKeyCode (XEvent *event)
 	case XK_Control_R :
 	  CONTROL = 0x00;
 	  break;
+	case XK_Caps_Lock :
+	  if (SHIFT)
+	    keycode = AKEY_CAPSLOCK;
+	  else
+	    keycode = AKEY_CAPSTOGGLE;
+	  break;
+	case XK_Shift_Lock :
+	  if (x11bug)
+	    printf ("XK_Shift_Lock\n");
+	  break;
 	default :
 	  break;
 	}
@@ -853,7 +504,7 @@ int GetKeyCode (XEvent *event)
   return keycode;
 }
 
-static int xview_keycode;
+static int xview_keycode = AKEY_NONE;
 
 #ifdef XVIEW
 
@@ -1156,12 +807,31 @@ void Atari_Initialise (int *argc, char *argv[])
 	windowsize = Large;
       else if (strcmp(argv[i],"-huge") == 0)
 	windowsize = Huge;
+      else if (strcmp(argv[i],"-x11bug") == 0)
+	x11bug = TRUE;
       else
-	argv[j++] = argv[i];
+	{
+	  if (strcmp(argv[i],"-help") == 0)
+	    {
+	      printf ("\t-small        Small window (%dx%d)\n",
+		      ATARI_WIDTH, ATARI_HEIGHT);
+	      printf ("\t-large        Large window (%dx%d)\n",
+		      ATARI_WIDTH*2, ATARI_HEIGHT*2);
+	      printf ("\t-huge         Huge window (%dx%d)\n",
+		      ATARI_WIDTH*3, ATARI_HEIGHT*3);
+	      printf ("\t-x11bug       Enable debug code in atari_x11.c\n");
+	    }
+
+	  argv[j++] = argv[i];
+	}
     }
 
   *argc = j;
 
+#ifdef NAS
+  NAS_Initialise (argc, argv);
+#endif
+
   switch (windowsize)
     {
     case Small :
@@ -1184,7 +854,6 @@ void Atari_Initialise (int *argc, char *argv[])
     {
       int status;
 
-      printf ("/dev/js0 is available\n");
       status = read (js0, &js_data, JS_RETURN);
       if (status != JS_RETURN)
 	{
@@ -1195,8 +864,9 @@ void Atari_Initialise (int *argc, char *argv[])
       js0_centre_x = js_data.x;
       js0_centre_y = js_data.y;
 
-      printf ("\tcentre_x = %d, centry_y = %d\n",
-	      js0_centre_x, js0_centre_y);
+      if (x11bug)
+	printf ("Joystick 0: centre_x = %d, centry_y = %d\n",
+		js0_centre_x, js0_centre_y);
 
       js0_mode = mode++;
     }
@@ -1206,7 +876,6 @@ void Atari_Initialise (int *argc, char *argv[])
     {
       int status;
 
-      printf ("/dev/js1 is available\n");
       status = read (js1, &js_data, JS_RETURN);
       if (status != JS_RETURN)
 	{
@@ -1217,8 +886,9 @@ void Atari_Initialise (int *argc, char *argv[])
       js1_centre_x = js_data.x;
       js1_centre_y = js_data.y;
 
-      printf ("\tcentre_x = %d, centry_y = %d\n",
-	      js1_centre_x, js1_centre_y);
+      if (x11bug)
+	printf ("Joystick 1: centre_x = %d, centry_y = %d\n",
+		js1_centre_x, js1_centre_y);
 
       js1_mode = mode++;
     }
@@ -1553,12 +1223,15 @@ void Atari_Initialise (int *argc, char *argv[])
 
   consol = 7;
 
-  printf ("Initial X11 controller configuration\n");
-  printf ("------------------------------------\n\n");
-  printf ("Keypad is "); Atari_WhatIs (keypad_mode); printf ("\n");
-  printf ("Mouse is "); Atari_WhatIs (mouse_mode); printf ("\n");
-  printf ("/dev/js0 is "); Atari_WhatIs (js0_mode); printf ("\n");
-  printf ("/dev/js1 is "); Atari_WhatIs (js1_mode); printf ("\n");
+  if (x11bug)
+    {
+      printf ("Initial X11 controller configuration\n");
+      printf ("------------------------------------\n\n");
+      printf ("Keypad is "); Atari_WhatIs (keypad_mode); printf ("\n");
+      printf ("Mouse is "); Atari_WhatIs (mouse_mode); printf ("\n");
+      printf ("/dev/js0 is "); Atari_WhatIs (js0_mode); printf ("\n");
+      printf ("/dev/js1 is "); Atari_WhatIs (js1_mode); printf ("\n");
+    }
 }
 
 int Atari_Exit (int run_monitor)
@@ -1588,6 +1261,10 @@ int Atari_Exit (int run_monitor)
       if (js1 != -1)
 	close (js1);
 #endif
+
+#ifdef NAS
+      NAS_Exit ();
+#endif
     }
 
   return restart;
@@ -1625,6 +1302,7 @@ void Atari_DisplayScreen (UBYTE *screen)
 
   int xpos;
   int ypos;
+  int i;
 
   consol = 7;
   keypad_trig = 1;
@@ -1706,6 +1384,10 @@ void Atari_DisplayScreen (UBYTE *screen)
   notify_dispatch ();
   XFlush (display);
 #endif
+
+#ifdef NAS
+  NAS_UpdateSound ();
+#endif
 }
 
 int Atari_Keyboard (void)
@@ -2021,6 +1703,7 @@ int Atari_CONSOL (void)
   return consol;
 }
 
+#ifndef NAS
 int Atari_AUDC (int channel, int byte)
 {
 }
@@ -2032,3 +1715,4 @@ int Atari_AUDF (int channel, int byte)
 int Atari_AUDCTL (int byte)
 {
 }
+#endif
